from os import path
import smtplib


class KindleEmailClient:
    def __init__(self, session, sender, receiver, abs_attachment_path=""):
        self.session = session
        self.sender = sender
        self.receiver = receiver
        self.abs_attachment_path = abs_attachment_path
        self.message = self.construct_empty_message()

    def send_email_with_book(self):
        self.attach_file_to_message()
        self.send_mail()

    def construct_empty_message(self):
        from email.mime.text import MIMEText
        from email.mime.multipart import MIMEMultipart

        message = MIMEMultipart()
        message["From"] = self.sender
        message["To"] = self.receiver
        message["Subject"] = "Book"
        message.attach(MIMEText("", "plain"))
        return message

    def attach_file_to_message(self):
        from email.mime.application import MIMEApplication

        filename = path.basename(self.abs_attachment_path)
        with open(self.abs_attachment_path, "rb") as file_reader:
            part = MIMEApplication(file_reader.read(), Name=filename)
            part["Content-Disposition"] = f'attachment; filename="{filename}"'
            self.message.attach(part)

    def send_mail(self):
        text = self.message.as_string()
        self.session.sendmail(self.sender, self.receiver, text)
        print("Mail Sent! \n")


def send_a_bunch_of_files_to_kindle(session_object, file_list, from_file, to_file):
    for abs_file_path in file_list:
        print(f"Attempting to e-mail {path.basename(abs_file_path)}")
        if not is_file_attachable(abs_file_path):
            print(f"Attachment too big: Skipping {abs_file_path}..")
            continue

        KindleEmailClient(
            session_object.session, from_file, to_file, abs_file_path
        ).send_email_with_book()

    session_object.session.quit()


def is_file_attachable(abs_file_path):
    from ..constants import gmail_attachment_threshold_mb

    file_size_in_mb = path.getsize(abs_file_path) / 1e6
    return file_size_in_mb < (gmail_attachment_threshold_mb - 2)  # 2 MB Buffer


class GmailSession:
    def __init__(self, sender):
        from smtplib import SMTPAuthenticationError
        from keyring import delete_password
        from keyring.errors import PasswordDeleteError

        self.sender = sender
        self.session = smtplib.SMTP("smtp.gmail.com", 587)
        self.session.starttls()

        try:
            passwd = self.get_password()
            self.session.login(self.sender, self.get_password())
        except SMTPAuthenticationError:
            try:
                print("Invalid credentials, deleting from keyring....")
                delete_password("system", self.sender)
            except PasswordDeleteError:
                print("ERROR: Keyring corrupted!")
            raise OSError

    def get_password(self):
        import keyring
        from getpass import getpass

        password = keyring.get_password("system", self.sender)
        if password is None:
            password = getpass(prompt=f"Enter e-mail password for {self.sender}: ")
            keyring.set_password("system", self.sender, password)
        return password



# Check if word is english
def is_word_english(word):
    import enchant

    return enchant.Dict("en_US").check(word)


# Check if token is helpful to find metadatas
def is_token_good(token):
    bad_tokens = ["ltd", "libgen", "org", "www", "com", "co"]
    if len(token) <= 3 and not is_word_english(token):
        return False
    if len(token) <= 1:
        return False
    if token in bad_tokens:
        return False
    return True


# Clean file names to help find metadata better
def clean_file_name(filename):
    import re

    extension = filename[filename.rfind(".") + 1 :]
    clean_name = filename[: filename.rfind(".")]
    clean_name = re.sub(r"[^A-Za-z\' ]+", " ", clean_name)
    clean_name = re.sub(r" +", " ", clean_name)
    clean_name = clean_name.strip().lower()
    clean_name = " ".join([word for word in clean_name.split() if is_token_good(word)])
    return clean_name, extension



from os import path, listdir


def get_commandline_args(args):
    from argparse import ArgumentParser

    parser = ArgumentParser()
    parser.add_argument(
        "--folder",
        help="folder to run conversion on (defaults to downloads if not specified)",
    )
    parser.add_argument(
        "--file",
        help="absolute file path for conversion, if you want a single file converted",
    )
    parser.add_argument(
        "--kindle",
        help="your Kindle email-id, books are auto-emailed to here after conversion",
    )
    parser.add_argument(
        "--email",
        help="your personal email-id, books are auto-emailed from here after conversion (use with --kindle for the first ever run, to store credentials)",
    )
    parser.add_argument(
        "--ext", help="extension to convert to (defaults to mobi if not specified)"
    )
    parser.add_argument(
        "--job",
        help="sets up a twice-a-day conversion job for specified folder (or downloads)",
        action="store_true",
    )
    parser.add_argument(
        "--force",
        help="Convert without checking if it is a book or within safe size threshold (only extension check is performed)",
        action="store_true",
    )
    args = parser.parse_args(args)
    return args


def check_commandline_args(args):
    from ..constants import valid_extensions_for_conversion
    import re

    is_dir = path.isdir(args.folder) if args.folder is not None else True
    is_kindle = (
        False
        if args.kindle is not None and not args.kindle.endswith("@kindle.com")
        else True
    )
    is_file = path.isfile(args.file) if args.file is not None else True
    is_extension = (
        False
        if args.ext is not None and args.ext not in valid_extensions_for_conversion
        else True
    )

    if not is_dir:
        return f"Error: {args.folder} is not an existing directory!"
    if not is_kindle:
        return f"Error: {args.kindle} is not a valid Kindle e-mail address!"
    if not is_file:
        return f"Error: {args.file} does not exist or is not a valid file!"
    if not is_extension:
        return f"Error: {args.ext} extension not present in {valid_extensions_for_conversion}"

    return None


def process_commandline_args(args):
    from .os_utils import get_downloads_folder_location

    if args.file is not None:
        file_list = [path.basename(args.file)]
        download_dir = path.split(args.file)[0]
        return file_list, download_dir
    if args.folder is not None:
        file_list = listdir(args.folder)
        return file_list, args.folder
    else:
        download_dir = get_downloads_folder_location()
        if download_dir is None:
            return None, None
        file_list = listdir(download_dir)
        return file_list, download_dir



# Finds your downloads location
from os import makedirs, path
from ..constants import file_appended_hash


def get_downloads_folder_location():
    try:  # GTK2
        import glib

        downloads_dir = glib.get_user_special_dir(glib.USER_DIRECTORY_DOWNLOAD)
    except (ModuleNotFoundError, AttributeError) as e:  # GTK3
        try:
            from pgi.repository import GLib

            downloads_dir = GLib.get_user_special_dir(
                GLib.UserDirectory.DIRECTORY_DOWNLOAD
            )
        except (ModuleNotFoundError, AttributeError) as e:  # GTK3
            downloads_dir = None
    return downloads_dir


def make_required_inodes(dir_list, file_list):
    for directory in dir_list:
        if not path.exists(directory):
            makedirs(directory)
    for file in file_list:
        if not path.exists(file):
            open(file, "a").close()


def name_required_inodes(folder):
    not_books_file = path.join(folder, f"not_books_{file_appended_hash}.txt")
    processed_dir = path.join(folder, f"Processed_Books_{file_appended_hash}")
    convert_dir = path.join(folder, f"Converted_Books_{file_appended_hash}")
    return not_books_file, processed_dir, convert_dir


def convert_file_to_list(file):
    try:
        line_list = open(file, "r").read().splitlines()
    except FileNotFoundError:
        line_list = []
    return line_list



from os import path


def trigger_conversion(absolute_file_path, processed_dir, convert_dir, extension):
    from subprocess import check_output
    from os import rename

    filename = path.basename(absolute_file_path)
    print(f"Beginning conversion: {filename}")
    converted_file_name = filename[: filename.rfind(".")] + "." + extension
    converted_file_path = path.join(convert_dir, converted_file_name)
    check_output(["ebook-convert", absolute_file_path, converted_file_path])
    rename(absolute_file_path, path.join(processed_dir, filename))
    print(f"Completed Conversion: {converted_file_name}")
    return converted_file_path



# Clean out all previous pykinldler cron jobs?
def remove_pykindler_cron_jobs(cron):
    for job in cron:
        if "pykindler-run" in str(job):
            cron.remove(job)
            print(f"Removing existing pykindler-run job: {str(job)}")
    cron.write()


# Make new job
def create_pykindler_cron_job(cron, args):

    command = "pykindler-run"
    command = command + f" --folder {args.folder}" if args.folder is not None else ""
    command = command + f" --email {args.email}" if args.email is not None else ""
    command = command + f" --ext {args.ext}" if args.ext is not None else ""
    command = command + f" --email {args.email}" if args.email is not None else ""
    command = command + f" --kindle {args.kindle}" if args.kindle is not None else ""
    command = command + f" --force" if args.force else ""
    print(f"Creating scheduled job {command}...")
    job = cron.new(command=command)
    job.hour.every(12)
    cron.write()


# Create the job, plus housekeeping
def setup_cron_job(args):
    from crontab import CronTab

    cron = CronTab(user=True)
    remove_pykindler_cron_jobs(cron)
    create_pykindler_cron_job(cron, args)
    print("Scheduled job created!")



conversion_threshold_in_mb = 10
gmail_attachment_threshold_mb = 25
valid_extensions_for_conversion = ["pdf", "djvu", "azw3", "epub", "mobi"]
extension_list = ["." + ext for ext in ["pdf", "epub", "djvu", "azw", "azw3", "mobi"]]
metadata_command = lambda x: ["fetch-ebook-metadata", "--opf", "--title", x]
file_appended_hash = "30234f2413d43e5c"



from os import path, rename
from subprocess import CalledProcessError, check_output
from .constants import *
from .utils.convert_utils import trigger_conversion
from .utils.os_utils import (
    make_required_inodes,
    name_required_inodes,
    convert_file_to_list,
)
from .utils.nlp_utils import clean_file_name


def process_and_convert_books(file_list, folder, args):
    print(f"Processing on folder: {folder}")
    not_books_file, processed_dir, convert_dir = name_required_inodes(folder)
    make_required_inodes([convert_dir, processed_dir], [not_books_file])

    not_book_list = convert_file_to_list(not_books_file)
    not_book_writer = open(not_books_file, "a")

    converted_files = []

    for filename in file_list:
        absolute_file_path = path.join(folder, filename)

        if args.force and True in set(
            filename.endswith(extension) for extension in extension_list
        ):
            converted_file_path = trigger_conversion(
                absolute_file_path, processed_dir, convert_dir, args.ext
            )
            converted_files.append(converted_file_path)
            continue

        # Move specified extension files as is
        if filename.endswith("." + args.ext):
            rename(absolute_file_path, path.join(convert_dir, filename))
            converted_files.append(path.join(convert_dir, filename))
            continue

        # If we looked at this file on last run, ignore
        if filename in not_book_list:
            print(f"File: {filename} ignored, as it was ignored on last run")
            continue

        # Ignore hidden files
        if filename.startswith("."):
            continue

        # Don't convert files which are too big
        file_size_in_mb = path.getsize(absolute_file_path) / 1e6
        if file_size_in_mb > conversion_threshold_in_mb:
            print(
                f"File: {filename} ignored, violates size threshold: {file_size_in_mb} MB > {conversion_threshold_in_mb} MB"
            )
            continue

        if True in set(filename.endswith(extension) for extension in extension_list):
            print(f"Looking at: {filename}")
            cleaned_file_name, ext = clean_file_name(filename)

            # Don't convert small word names, metadata search messes up these
            if len(cleaned_file_name.split()) <= 2:
                print("File name has less than 3 words, ignoring...")
                continue

            # Filter out non-books using calibre's metadata bash calls
            try:
                # run(metadata_command(cleaned_file_name), check=True)
                check_output(metadata_command(cleaned_file_name))
                converted_file_path = trigger_conversion(
                    absolute_file_path, processed_dir, convert_dir, args.ext
                )
                converted_files.append(converted_file_path)
            except CalledProcessError:
                print(f"Not a book: {filename}")
                not_book_writer.write(filename + "\n")
                continue

    # Cleanup
    not_book_writer.close()

    print(
        f"Finished processing folder: {folder}.\n Please check folders {processed_dir} and {convert_dir} for your books! "
    )

    return converted_files



#!/usr/bin/env python
import sys
from pykindler.utils.bash_utils import (
    get_commandline_args,
    check_commandline_args,
    process_commandline_args,
)
from .utils.cron_utils import setup_cron_job
from .utils.email_utils import *
from .convertor import process_and_convert_books
from os import path


def client():
    args = get_commandline_args(sys.argv[1:])
    error_message = check_commandline_args(args)
    if error_message is not None:
        print(error_message)
        sys.exit(2)

    file_list, download_dir = process_commandline_args(args)
    if file_list is None and download_dir is None:
        print("Error: Unable to detect downloads folder, please specify --folder")
        sys.exit(2)

    if args.job is True:
        setup_cron_job(args)

    if args.ext is None:
        args.ext = "mobi"

    if bool(args.email is None) != bool(args.kindle is None):
        print("Error: To auto-email, please provide both --email and --kindle")
        sys.exit(2)

    try:
        session_object = GmailSession(args.email) if args.email is not None else None
        print("Credentials are valid! Books will be e-mailed after conversion...\n")
    except OSError:
        print("Error: Please enter valid e-mail credentials!")
        sys.exit(2)

    converted_file_list = process_and_convert_books(file_list, download_dir, args)
    if args.email is not None:
        print("\n Auto-emailing books to Kindle...")
        send_a_bunch_of_files_to_kindle(
            session_object, converted_file_list, args.email, args.kindle
        )

    print("Exiting...")
    sys.exit()



__author__ = "Vivek Nayak"

import setuptools
from os import path
from pykindler import cli

try:
    # Used to convert md to rst for pypi, otherwise not needed
    import m2r
except ImportError:
    m2r = None

thelibFolder = path.dirname(path.realpath(__file__))
requirementPath = thelibFolder + "/requirements.txt"
install_requires = []
if path.isfile(requirementPath):
    with open(requirementPath) as f:
        install_requires = f.read().splitlines()

description = "Schedules conversion of e-books to mobi, mails it to your kindle"
if m2r is None:
    long_description = description
else:
    # convert markdown to rst
    long_description = m2r.convert(open("README.md").read())

setuptools.setup(
    name="pykindler",
    install_requires=install_requires,
    version="0.3.2",
    description=description,
    long_description=long_description,
    license="MIT",
    author="Vivek Nayak",
    author_email="viveknayak2210@gmail.com",
    url="https://github.com/VivekBits2210/pykindler",
    entry_points={"console_scripts": ["pykindler-run=pykindler.cli:client"]},
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: End Users/Desktop",
        "Environment :: Console",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: MIT License",
        "Topic :: Utilities",
    ],
)
